#!/usr/local/bin/perl -w

###########################################################################
#
# replace.pl version 0.1
#
###########################################################################
#
# This program may be used to replace the contexts of certain instances
# in a senseval-2 data format SOURCE file with modified/new contexts. 
# The instances whose contexts are to be replaced and the modified/new 
# contexts are specified via a CONTEXT file. The CONTEXT file must have 
# the instance ID of the instance whose context is to be replaced followed
# by the modified/new context. The context must be demarcated by <context>
# and </context> tags. The instance ID, <context> and <\/context> tags must 
# be on new lines. Blank lines are allowed, however, everything 
# between the context tags is considered part of context.
# The token(s) between the <head> and </head> tags of the context in the 
# CONTEXT file are replaced by correponding tokens in the SOURCE file.
# If the original context has its head word pre-tagged with a part of 
# speech, the updated context will thus have the pre-tag. 
# The updated SOURCE is placed in the DESTINATION file.
# 
# USAGE :  replace.pl OPTIONS CONTEXT DESTINATION SOURCE
#
#	--version       Prints the version number.
#	--help		Prints help message.
# 
###########################################################################
#
# Copyright (C) 2000-2002
#
# Mohammad Saif, University of Minnesota, Duluth
# moha0149@d.umn.edu
#
# Ted Pedersen, University of Minnesota, Duluth
# tpederse@d.umn.edu
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
###########################################################################

# START OF PROGRAM
use Getopt::Long;

# Check if any options specified
GetOptions("version", "help");
if ( defined $opt_help )    { $help = $opt_help; }
else                        { $help = 0; }
# Print help if asked
if ( $help ) {
    &help();
    exit; }

# if version has been requested, show version
if ( defined $opt_version ) {
    $opt_version = 1;
    &showVersion();
    exit; }

# Check if command line options have been provided
# If not print usage information
if ( $#ARGV < 2 ) {
    &usage();
    exit;}

$filter = shift();
$opfl = shift();
$inpfl = shift();

if(-f $opfl) {
    print "Destination file already exists. Press `y' to overwrite\n";
    $resp = <STDIN>;
    chomp($resp);
    if($resp eq "y" || $resp eq "Y") {
        print "Overwriting the file.\n"; }
    else {
        exit; } }

if(!(-f $filter)) {
    print "CONTEXT file does not exist.\n";
    exit; }

if(!(-f $inpfl)) {
    print "SOURCE file does not exist.\n";
    exit; }

# Open file
open(INP,"<$inpfl");
open(DB,"<$filter");
open(OP,">$opfl");

my %con;
my $id;
my $ln;
# Read lines from CONTEXT file
while($ln = <DB>) {
    if($ln =~ /^\s*$/)	{ next; }

    ### CAPTURE INSTANCE ID INFORMATION
    if($ln =~ /^\s*([^\s].*[^\s])\s*$/) {
	$id = $1;
	# Capture instance ID
	$instH{$id}++;
	if($instH{$id} > 1)	{
		print "Duplicate instance ID.\n";
		exit;
	}
   

    while($ln = <DB>) {
        if($ln !~ /^\s*$/)  { last; } }
    ### CAPTURE CONTEXT
 
    if($ln =~ /^\s*<context>\s*$/) {
	$ctr = 0;
	# capture context
	while($cline = <DB>) {
	    if($cline =~ /^\s*<\/context>\s*$/)  { last; }
	    $con{$id}{$ctr} = $cline;	
	    $ctr++;
	}
     }

     }
}

### READ SOURCE FILE
while($ln = <INP>) {
    ### CAPTURE INSTANCE ID INFORMATION
    if($ln =~ /^\s*<instance id=\"([^\"]+)\"/) {
        $id = $1;
	print OP $ln;
	if(!(exists $con{$id}{0}))	{ next; }

	# INSTANCE ID found
	# Print details till <context> from SOURCE
	while($ln = <INP>) {
		print OP $ln;
		if($ln =~ /^\s*<context>\s*$/) 	{ last; }
	}

	$hdwd = "";
	# skip old context
        while($ln = <INP>) {
		if($ln =~ /<head[^>]*>([^<]+)<\/head>/)	{ $hdwd = $1; }
                if($ln =~ /^\s*<\/context>\s*$/)  { last; }
        }
	if($hdwd eq "")	{
		print "no head word in the instance $id of CONTEXT file.\n";
		exit;
	}

	# Print updated/new context
	$ctr=0;
	while(exists $con{$id}{$ctr}) {
		# Check if line has head word
		if($con{$id}{$ctr} =~ /<head[^>]*>[^<]+<\/head>/) {
			$con{$id}{$ctr} =~ s/(<head[^>]*>)[^<]+(<\/head>)/$1$hdwd$2/;
		}
		print OP "$con{$id}{$ctr}";
		$ctr++;
	}

    }
    print OP $ln;
}

# Subroutine to print help message
sub help{
    print "Usage: replace.pl OPTIONS CONTEXT DESTINATION SOURCE\n\n"; 
    print "This program may be used to replace the contexts of certain instances. The instances whose contexts are to be replaced and the modified contexts are specified via a CONTEXT file. The CONTEXT file must have the instance ID of the instance whose context is to be replaced followed by the modified context. The context must be demarcated by <context> and <\/context> tags. The instance ID, <context> and <\/context> tags must be on new lines.\n\n";
    print "OPTIONS:\n\n";
    print "  --version    Prints the version number.\n";
    print "  --help       Prints the help message.\n\n";}

sub usage{
    print "Usage: replace.pl OPTIONS CONTEXT DESTINATION SOURCE\n"; 
    AskHelp();}
# Message describing way to get help
sub AskHelp{
    print "Type replace.pl --help for help.\n";}

# function to output the version number
sub showVersion
{
    print STDERR "replace.pl      -        version 0.1\n";
    print STDERR "A component of refine  0.1\n";
    print STDERR "Copyright (C) 2000-2003, Saif Mohammad and Ted Pedersen\n";
}
